/*
	File:		CColor.h

	Contains:	Interface file for a QuickDraw layer class

	Written by:	Arno Gourdol

	Copyright:	 1994-1995 by Apple Computer, Inc., all rights reserved.

*/

#pragma once

#ifndef __CCOLOR__
#define __CCOLOR__

#include "assert.h"

#include <Quickdraw.h>

typedef enum ColorIndex 
{
	kCurrentForeColor = 0,
	kWhiteColor = 30000,
	kBlackColor = 30001,
	kInsetBoxShadowColor = 30004,
	kInsetBoxHiliteColor = 30005,
	kWindowBackgroundColor = 30006,
	kBevelShadowColor = 30007,
	kBevelHiliteColor = 30008,
	kBevelFrameColor = 30009,
	kBevelBackgroundColor = 30010,
	kInvBevelFrameColor = 30011,
	kTextFrameFillColor = 30012,
	kFocusRingColor = 30013,
	kHighlightColor = 30014
} ColorIndex;



class CColor
{
public:
	// constructors
	CColor(ColorIndex colorIndex);
	inline CColor(UInt32 red, UInt32 green, UInt32 blue);
	inline CColor(UInt32 gray);
	
	// destructor
	inline ~CColor();
	
	// operators
	inline operator RGBColor*();
	inline operator const RGBColor*() const;
	inline operator RGBColor&();

	// Accessors
	inline UInt32 Red(void) const;
	inline UInt32 Green(void) const;
	inline UInt32 Blue(void) const;

	// predicates
	inline Boolean IsBlack(void) const;
	inline Boolean IsWhite(void) const;
	
	
private:
	RGBColor fColor;
};

inline CColor::CColor(UInt32 red, UInt32 green, UInt32 blue)
{
	fColor.red = red;
	fColor.green = green;
	fColor.blue = blue;
}

inline CColor::CColor(UInt32 gray)
{
	fColor.red = gray;
	fColor.green = gray;
	fColor.blue = gray;
}

inline CColor::~CColor()
{

}

inline CColor::operator RGBColor*()
{
	return &fColor;
}

inline CColor::operator const RGBColor*() const
{
	return (RGBColor* const)&fColor;
}

inline CColor::operator RGBColor&()
{
	return fColor;
}

inline UInt32 CColor::Red(void) const
{
	return fColor.red;
}

inline UInt32 CColor::Green(void) const
{
	return fColor.green;
}

inline UInt32 CColor::Blue(void) const
{
	return fColor.blue;
}


inline Boolean CColor::IsBlack(void) const
{
	return (fColor.red == 0 && fColor.green == 0 && fColor.blue == 0);
}


inline Boolean CColor::IsWhite(void) const
{
	return (fColor.red == 0xFFFF && fColor.green == 0xFFFF && fColor.blue == 0xFFFF);
}



#endif

